@@ -3,14 +3,14 @@ Copyright 2020 The Microsoft DeepSpeed Team
 '''
 import json
 import math
-import importlib
 import torch
-from torch import nn
 from torch.autograd import Function
-import time
 from ... import op_builder
 import torch.nn as nn
 from deepspeed import comm as dist
+from deepspeed.utils.logging import log_dist
+from deepspeed.utils.types import ActivationFuncType
+
 # Cuda modules will be imported if needed
 inference_cuda_module = None
 minus_inf = -10000.0
@@ -72,6 +72,7 @@ class DeepSpeedInferenceConfig(TransformerConfig):
                  rotate_every_two=True,
                  return_tuple=True,
                  mlp_after_attn=True,
+                 mlp_act_func_type=ActivationFuncType.GELU,
                  training_mp_size=1,
                  bigscience_bloom=False):
         super(DeepSpeedInferenceConfig,
@@ -96,6 +97,7 @@ class DeepSpeedInferenceConfig(TransformerConfig):
         self.rotate_every_two = rotate_every_two
         self.return_tuple = return_tuple
         self.mlp_after_attn = mlp_after_attn
+        self.mlp_act_func_type = mlp_act_func_type
         self.specialized_mode = False
         self.training_mp_size = training_mp_size
         self.bigscience_bloom = bigscience_bloom
@@ -192,6 +194,7 @@ class DeepSpeedSelfAttentionFunction(Function):
             return tensor_list
 
         def backup_attention(mixed_x_layer, layer_past, alibi, input_mask, norm_factor):
+            alibi = alibi.to(torch.cuda.current_device())
             head_dim = hidden_size_per_partition // num_attention_heads_per_partition
             new_tensor_shape = mixed_x_layer.size()[:-1] + (
                 num_attention_heads_per_partition,
@@ -336,7 +339,7 @@ class DeepSpeedSelfAttentionFunction(Function):
                         torch.empty(1),
                         num_attention_heads_per_partition,
                         (1 / norm_factor if config.scale_attention else 1.0),
-                        (not unfused_mode),
+                        (not unfused_mode),  # noqa: F821
                         config.triangular_masking,
                         config.local_attention,
                         config.window_size,
@@ -344,21 +347,21 @@ class DeepSpeedSelfAttentionFunction(Function):
                 else:
                     attn_key_value = score_context_func(
                         mixed_query,
-                        (key_layer if unfused_mode else past_key.type_as(key_layer)),
+                        (key_layer if unfused_mode else past_key.type_as(key_layer)),  # noqa: F821
                         key_layer,
                         ((1 - input_mask).half() *
                          minus_inf) if input_mask.dtype == torch.int64 else input_mask,
                         (value_layer
-                         if unfused_mode else past_value.type_as(value_layer)),
+                         if unfused_mode else past_value.type_as(value_layer)),  # noqa: F821
                         value_layer,
                         num_attention_heads_per_partition,
                         (1 / norm_factor if config.scale_attention else 1.0),
-                        (not unfused_mode),
+                        (not unfused_mode),  # noqa: F821
                         config.triangular_masking,
                         config.local_attention,
                         config.window_size,
                         no_masking)
-                if unfused_mode:
+                if unfused_mode:  # noqa: F821
                     context_layer, _, _ = attn_key_value
                 else:
                     context_layer, key_layer, value_layer = attn_key_value
@@ -589,7 +592,8 @@ class DeepSpeedMLPFunction(Function):
                 mlp_gemm_func,
                 fused_gemm_gelu,
                 vector_matmul_func,
-                bias_residual_func):
+                bias_residual_func,
+                activation_func_type=ActivationFuncType.GELU):
 
         if config.q_int8:
             (intermediate,
@@ -629,7 +633,8 @@ class DeepSpeedMLPFunction(Function):
                                              attn_nb,
                                              config.epsilon,
                                              config.pre_layer_norm,
-                                             config.mlp_after_attn)
+                                             config.mlp_after_attn,
+                                             config.mlp_act_func_type)
                 output = vector_matmul_func(intermediate, output_w, False)
 
         inference_cuda_module.residual_add(
@@ -768,7 +773,8 @@ class DeepSpeedTransformerInference(nn.Module):
             builder = op_builder.InferenceBuilder()
             inference_cuda_module = builder.load()
 
-        print("DeepSpeed Transformer Inference config is ", self.config.__dict__)
+        if DeepSpeedTransformerInference.layer_id == 1:
+            log_dist(f"DeepSpeed-Inference config: {self.config.__dict__}", [0])
 
         self.attention = DeepSpeedSelfAttention(self.config,
                                                 mp_group,
@@ -794,24 +800,35 @@ class DeepSpeedTransformerInference(nn.Module):
                         device=device))
         self.layer_past = None
 
-    def forward(self,
-                input,
-                input_mask=None,
-                attention_mask=None,
-                head_mask=None,
-                layer_past=None,
-                get_key_value=False,
-                get_present=False,
-                encoder_output=None,
-                enc_dec_attn_mask=None,
-                encoder_hidden_states=None,
-                encoder_attention_mask=None,
-                use_cache=False,
-                alibi=None,
-                output_attentions=False):
+    def forward(
+            self,
+            input,
+            input_mask=None,
+            attention_mask=None,
+            head_mask=None,
+            layer_past=None,
+            get_key_value=False,
+            get_present=False,
+            encoder_output=None,
+            enc_dec_attn_mask=None,
+            encoder_hidden_states=None,
+            encoder_attention_mask=None,
+            use_cache=False,
+            alibi=None,
+            output_attentions=False,
+            # TODO(arashb): 'layer_head_mask' and 'past_key_value' are only added to satisfy the OPT models API.
+            # This needs to be redesigned later!
+            layer_head_mask=None,
+            past_key_value=None):
         get_present = (get_present or get_key_value or use_cache)
         input_mask = input_mask if attention_mask is None else attention_mask
+
+        # We set the prev key/value to None when there is a prompt
+        if input.shape[1] > 1:
+            self.layer_past = None
+
         layer_past = layer_past if layer_past is not None else self.layer_past
+        head_mask = layer_head_mask if layer_head_mask is not None else head_mask
 
         attn_mask = None
         if isinstance(input, tuple):
