@@ -3,13 +3,9 @@ Copyright 2020 The Microsoft DeepSpeed Team
 '''
 import types
 import torch
-import importlib
 import numpy as np
-import time
 from deepspeed import comm as dist
 
-from deepspeed.utils.logging import logger
-
 
 class OnebitAdam(torch.optim.Optimizer):
     """Implements the 1-bit Adam algorithm. Currently GPU-only.
@@ -37,7 +33,7 @@ class OnebitAdam(torch.optim.Optimizer):
         cuda_aware (boolean, required): Set True if the underlying MPI implementation
             supports CUDA-Aware communication. (default: False)
         comm_backend_name (string, optional): Set to 'mpi' if needed. (default: 'nccl')
-    .. _Adam\: A Method for Stochastic Optimization:
+    .. _Adam\\: A Method for Stochastic Optimization:
         https://arxiv.org/abs/1412.6980
     .. _On the Convergence of Adam and Beyond:
         https://openreview.net/forum?id=ryQu7f-RZ
